/**
 *
 * ADTF Tutorial filter for data processing
 *
 */
#pragma once

// Include all necessary headers from the ADTF SDK
#include <adtffiltersdk/adtf_filtersdk.h>

// For simplicity use the necessary namespaces
using namespace adtf::util;
using namespace adtf::ucom;
using namespace adtf::base;
using namespace adtf::streaming;
using namespace adtf::mediadescription;
using namespace adtf::filter;

// Now we declare a new class cTutorialFilterDataGenerator that
// inherits from cFilter. This will be our Tutorial Filter.
class cTutorialFilterDataGenerator: public cFilter
{
public:
    // Inside the class declaration we use an ADTF macro to enable correct treatment
    // of class identifier and class name by IClassInfo and the class factories.
    ADTF_CLASS_ID_NAME(cTutorialFilterDataGenerator,
                       "tutorial_data_generator.filter.adtf_guides.cid",
                       "Tutorial Data Generator");

public:
    // We implement the constructor where we create our Pins and Runners.
    cTutorialFilterDataGenerator();

    // We override the Process method in order to handle triggers from the
    // connected Active Runner.
    tResult Process(tNanoSeconds tmTrigger, IRunner* pRunner) override;

private:
    // Some helper methods.
    float CalculateVelocityValue();
    float CalculateYawRateValue();

    // Our writers for our output data.
    ISampleWriter* m_pVelocityWriter;
    ISampleWriter* m_pYawRateWriter;

    // Defines property variables, this properties will occur within your property set
    // You still need to register with name in the constructor.
    // Defines the change rates for the velocity/yaw rate values.
    property_variable<float> m_fVelocityChangeRate = 5.0;
    property_variable<float> m_fYawRateChangeRate = 1.0;
    // Defines the max values for velocity and yaw rate.
    property_variable<float> m_fVelocityMax = 100.0;
    property_variable<float> m_fYawRateMax = 10.0;

    // Store the current values.
    float m_fCurrentVelocity = 50.0;
    float m_fCurrentYawRate = 10.0;
};
